@@ -20,6 +20,9 @@ class Device():
         self.executionId = None
         self.taskId = None
 
+        PNPdevice = None
+        INVdevice = None
+
         if 'id' in kwargs:
             self.id = kwargs['id']
             self.initMethod = 'id'
@@ -30,82 +33,85 @@ class Device():
             self.hostname = kwargs['hostname']
             self.initMethod = 'hostname'
 
+        # Make sure at least one argument is set to find the device
         if self.id is None and self.serialNumber is None and self.hostname is None:
             raise ezDNACError(
                 'No device argment found. Enter hostname, id or sn')
 
-        # if init method is id, the device must be in inventory. Populate all attributes:
-        if self.initMethod == 'id':
-            try:
-                self.state = "Provisioned"
-                INVdevices = self.dnac.getInventoryDevies(id=self.id)
-            except:
+        # if init method is id, the device must be in inventory. 
+        if self.initMethod in 'id':
+
+            self.state = "Provisioned"
+            INVdevice = self.dnac.getInventoryDevies(id=self.id)
+            if INVdevice == None:
                 raise ezDNACError('device not found by id')
 
+        # if init method is hostname, the device must be in inventory. 
         elif self.initMethod == 'hostname':
-            try:
-                INVdevices = self.dnac.getInventoryDevies(
-                    hostname=self.hostname)
-            except:
+
+            INVdevice = self.dnac.getInventoryDevies(hostname=self.hostname)
+            if INVdevice == None:
                 raise ezDNACError('device not found by hostname')
 
         # if method is sn, the device can be either in inventory or pnp, have to try both
         elif self.initMethod == 'sn':
 
             # Try inventory:
-            INVdevices = self.dnac.getInventoryDevies(sn=self.serialNumber)
+            INVdevice = self.dnac.getInventoryDevies(sn=self.serialNumber)
 
-            # If any data found in inventory, populate based on that:
-            if INVdevices is not None:
-                self.state = "Provisioned"
-                self.id = INVdevices['id']
-                self.serialNumber = INVdevices['serialNumber']
-                self.ip = INVdevices['managementIpAddress']
-                self.hostname = INVdevices['hostname']
-                self.platform = INVdevices['platformId']
-                self.softwareVersion = INVdevices['softwareVersion']
-                self.softwareType = INVdevices['softwareType']
-                self.collectionStatus = INVdevices['collectionStatus']
+            if INVdevice == None:
+                PNPdevice = self.dnac.getPnpDevices(sn=self.serialNumber)
+
+                if PNPdevice == None:
+                    raise ezDNACError('device not found by serial Number')
+
+        if INVdevice is not None:
+            self.state = "Provisioned"
+            self.id = INVdevice['id']
+            self.platform = INVdevice['platformId']
+            for key, value in INVdevice.items():
+                setattr(self, key, value)
 
 
-        if self.state != "Provisioned" and self.initMethod == 'sn':
-            try:
-                # Otherwise try populate attributes via pnp:
-                PNPdevices = self.dnac.getPnpDevices(sn=self.serialNumber)
+        # If device is found in pnp inventory, populate it
+        if PNPdevice is not None:
+            self.id = PNPdevice['id']
+            for key, value in PNPdevice['deviceInfo'].items():
+                setattr(self, key, value)
 
-                # If stack, check first switch
-                if 'stackInfo' in PNPdevices['deviceInfo']:
-                    if 'stackMemberList' in PNPdevices['deviceInfo']['stackInfo']:
-                        if len(PNPdevices['deviceInfo']['stackInfo']['stackMemberList']) > 1:
-                            
-                            self.hostname = PNPdevices['deviceInfo']['name']
-                            self.id = PNPdevices['id']
-                            self.softwareType = PNPdevices['deviceInfo']['agentType']
-                            self.state = PNPdevices['deviceInfo']['state']
-                            self.platform = PNPdevices['deviceInfo']['pid']
 
-                            # Some attributes is picked from first switch in stack.                            
-                            stack1 = PNPdevices['deviceInfo']['stackInfo']['stackMemberList'][0]
-                            self.softwareVersion = stack1['softwareVersion']       
+            # If stack, check first switch
+            if 'stackInfo' in PNPdevice['deviceInfo']:
+                if 'stackMemberList' in PNPdevice['deviceInfo']['stackInfo']:
+                    if len(PNPdevice['deviceInfo']['stackInfo']['stackMemberList']) > 1:
+                        
+                        self.hostname = PNPdevice['deviceInfo']['name']
+                        self.id = PNPdevice['id']
+                        self.softwareType = PNPdevice['deviceInfo']['agentType']
+                        self.state = PNPdevice['deviceInfo']['state']
+                        self.platform = PNPdevice['deviceInfo']['pid']
+
+                        # Some attributes is picked from first switch in stack.                            
+                        stack1 = PNPdevice['deviceInfo']['stackInfo']['stackMemberList'][0]
+                        self.softwareVersion = stack1['softwareVersion']       
 
                 else:
-                    self.id = PNPdevices['id']
-                    self.state = PNPdevices['deviceInfo']['state']
-                    self.hostname = PNPdevices['deviceInfo']['name']
-                    self.platform = PNPdevices['deviceInfo']['pid']
-                    self.softwareVersion = PNPdevices['deviceInfo']['imageVersion']
-                    self.softwareType = PNPdevices['deviceInfo']['agentType']
+                    self.id = PNPdevice['id']
+                    self.state = PNPdevice['deviceInfo']['state']
+                    self.hostname = PNPdevice['deviceInfo']['name']
+                    self.platform = PNPdevice['deviceInfo']['pid']
+                    self.softwareVersion = PNPdevice['deviceInfo']['imageVersion']
+                    self.softwareType = PNPdevice['deviceInfo']['agentType']
+
+
+                try:
+                    httpHeaders = PNPdevice['deviceInfo']['httpHeaders']
+                    for header in httpHeaders:
+                        if header['key'] == 'clientAddress':
+                            self.ip = header['value']
+                except:
+                    pass
 
-            except:
-                raise NotFound('device with serialNumber ' +
-                                  str(self.serialNumber) + ' not found')
-            try:
-                httpHeaders = PNPdevices['deviceInfo']['httpHeaders']
-                for header in httpHeaders:
-                    if header['key'] == 'clientAddress':
-                        self.ip = header['value']
-            except:
-                pass
 
     def updateAttributes(self):
         endpoint = f"network-device/{self.id}"
@@ -199,6 +205,7 @@ class Device():
 
         response = restcall('POST', self.dnac, endpoint, jsondata=payload)
 
+        
         # If error occurs, no id
         if 'response' in response and 'errorCode' in response['response']:
             data['deploymentId'] = None
